@--------------- 
this file contains the procs that will be used in program
mcpower.gra and mcsign.prg
some procs may be adjusted by Wei and hence different from 
the original ones from Alex
--------------------------------@


/****** sign1 *******
sign1:  simple sign tests
y_k     : matrix of k period returns. Each col. corresponds
          to a k value in _k. entry i,j is given by
	  lrd_{t+1}+...+lrd_{t+_k_{j}}
	  Note: y_k will have some missing values towards the top 
x       : matrix of regressor (one col. of x for each col. of y_k (no constant included))
output:   
res     : 3 by rows(_m) matrix of regression output. Each column is output for 1 regression
          including  100*Sg/n| 100*se/n|p-val, where  100*Sg/n is percent of correctly signed 
          predictions, 100*se/n is the standard, p-val is the two sided p-value
********/

proc sign1(_x,y_k);
local xx,yy,iT,b,S,tb,R2,u,m,datai,_xy,res;  
local xdm,ydm,Sg,n,se,pr;
if rows(_m) == cols(y_k) and cols(y_k) == cols(_x);
else;
   "sign1: Error: rows(_m) NOT= cols(y_k) or cols(y_k) NOT= cols(x)";
  end;
endif;
res=zeros(3,rows(_m));   /*save res=results*/
datai = 1;
do until datai>rows(_m);
  _xy = packr(_x[.,datai]~y_k[.,datai]); 
  @Remove missing values. Note: series are column by column.@
 /*_xy = _x~y_k[.,1];*/
  xx = _xy[.,1];   
  yy = _xy[.,2];
  m = _m[datai];
  iT= rows(yy);
  /*xdm = xx-median(xx);*/
  ydm = yy-median(yy);
  xdm = med2(xx); @removes the median of x using only past x@
 /* ydm = med2(yy); original */
 {Sg,n,se,pr} = signstat(ydm,xdm,0);
  res[.,datai]= 100*Sg/n| 100*se/n|pr;
  datai = datai+1;
endo;
retp(res);
endp;
@***********@


proc signrnk1(_x,y_k);
local xx,yy,iT,b,S,tb,R2,u,m,datai,_xy,res;  
local xdm, ydm,Srg,n,se,pr;
if rows(_m) == cols(y_k) and cols(y_k) == cols(_x);
else;
   "sign1: Error: rows(_m) NOT= cols(y_k) or cols(y_k) NOT= cols(x)";
  end;
endif;
res=zeros(1,rows(_m)); @save res=results@
datai = 1;
do until datai>rows(_m);
  _xy = packr(_x[.,datai]~y_k[.,datai]);  
  @Remove missing values. Note: series are column by column.@
 /* _xy = _x~y_k[.,1];*/
  xx = _xy[.,1];   
  yy = _xy[.,2];
  m = _m[datai];
  iT= rows(yy);
  /*xdm = xx-median(xx);*/
  ydm = yy-median(yy);
  xdm = med2(xx); @removes the median of x using only past x@
 /* ydm = med2(yy); original */
 {Srg} = signrank(ydm,xdm,0);
  
  /*res[.,datai]= 100*Sg/n| 100*se/n|pr;*/
  
  n = rows(xdm);
  local pv;
  if Srg < n*(n+1)/4;
  pv = cdfn((Srg - n*(n+1)/4)/sqrt(n*(n+1)*(2*n+1)/24));
  else;
  pv = 1-cdfn((Srg - n*(n+1)/4)/sqrt(n*(n+1)*(2*n+1)/24));
  endif;
  res[.,datai] = pv;
  datai = datai+1;
endo;
retp(res);
endp;
@***********@


/***mkkrep: make k repitions ****
inputs: y:       N by 1 data series
numrep:  number of repitions needed
output: ykrep:   N by numrep Matrix. 
All columns are equal to y, which has been repeated numrep times
*****/		 
proc(1) = mkkrep(y,numrep);
local r,ykrep;
ykrep = y;
r = 2;
do until r>numrep;
  ykrep = ykrep~y;
  r =r+1;
endo;
retp(ykrep);
endp;


/****** hacreg *******
hackkern:  string of kernel in HAC estimation. Choices are:
            "nw" for Newey-West or "hh" for Hansen-Hodrick 
_m      : vector of bandwidths for HAC estimator (rows(_m) = cols(y_k<));
          Campbell sets _m to _k-1 where k is the vector of horizons
y_k     : matrix of k period returns. Each col. corresponds
          to a k value in _k. entry i,j is given by
	  lrd_{t+1}+...+lrd_{t+_k_{j}}
	  Note: y_k will have some missing values towards the top 
x       : matrix of regressor (one col. of x for each col. of y_k (no constant included))
output:   
res     : 3 by rows(_m) matrix of regression output. Each column is output for 1 regression
          including b|R2|tb, where tb is calculated using hac covariance. setting _m=0
	  gives standard (non-hac) t-statistic.
********/
proc hacreg(_x,y_k,_m,hackern);
local xx,yy,iT,b,S,tb,R2,u,m,datai,_xy,res;

if rows(_m) == cols(y_k) and cols(y_k) == cols(_x);
else;
   "lrheg: Error: rows(_m) NOT= cols(y_k) or cols(y_k) NOT= cols(x)";
  end;
endif;
res=zeros(3,rows(_m)); @save results@
datai = 1;
do until datai>rows(_m);
   
  _xy = packr(_x[.,datai]~y_k[.,datai]);  
  @Remove missing values. Note: series are column by column.@
  
  xx = _xy[.,1]; 
   
  yy = _xy[.,2];
  yy = yy - meanc(yy); @demean@
  xx = xx - meanc(xx); @demean@
  m = _m[datai];
  iT= rows(yy);
  b=inv(xx'xx)*xx'yy;
  u=yy-xx*b;
  if hackern $== "nw";
    S=hacnw(xx,u,m);
  elseif hackern $== "hh";
    S=hachh(xx,u,m);
  endif;
    tb=b./sqrt(diag(iT*inv(xx'xx)*S*inv(xx'xx)));
    R2 = (stdc(xx*b)/stdc(yy))^2; 
/*
" xxb ";; iscplx(xx*b);; imag(xx*b);
" xx ";; iscplx(xx);; imag(xx);
" b ";; iscplx(b);; imag(b);
wait;
*/

    if _m[datai] == 0; @Remove to replicate exactly for k=1@
      tb=b./sqrt(diag(inv(xx'xx)*1/(iT-2)*u'u ));
    endif; 
  res[.,datai]=b|R2|tb;
  datai = datai+1;
endo;
retp(res);
endp;
@***********@


/***** proc to calculate the size corrected critical value*****
imput: simulated xt-1 and yt, desired size level, and horizons
output: the absolute critical values giving the desired size level
***************************************************************/

proc(1) = sizecorr(y,x,alpha,_k);
local tbols,i,yt,xt,yy,xx,tau,s,frac,taubound,j,bound,xypack; 
local nam,m,b,stb,vc,std,sig,cx,rsq,resid,dbw;

bound = 0;
for j (1, rows(_k), 1);


tbols = zeros(1,cols(x));
for i (1,cols(tbols),1);

yt = y[.,i];
xt = mkkpslx(x[.,i],_k[j]);
xypack = packr(yt~xt);
yy = xypack[.,1];
xx = xypack[.,2];


{ nam,m,b,stb,vc,std,sig,cx,rsq,resid,dbw } = ols(0,yy,xx); 


tbols[1,i] = b[2]/std[2];
endfor;

tau = sortc(tbols',1);
if abs(tau[1])>=abs(tau[rows(tau)]);
for s (1, cols(tbols), 1);
taubound = abs(tau[s]);
frac = sumc((abs(tbols).>taubound)')/cols(tbols);
  if abs(frac-0.05) <= 1e-6; 
  break;
  endif;
endfor;

else;
for s (1, cols(tbols), 1);
taubound = abs(tau[rows(tau)+1-s]);
frac = sumc((abs(tbols).> taubound)')/cols(tbols);
  if abs(frac-0.05) <= 1e-6; 
  break;
  endif;
endfor;

endif;

bound = bound|taubound;
endfor;
retp(bound[2:rows(bound)]);
endp;

 
/********
mkkpslx: make k-period sum of lag Xs.
 input: 
    x:  N by 1 vector x[t] = x_t
    _k: vector of lag values (_k-1) is number of lags (integer)
 output:
    kpslx: N by rows(_k) matrix of k period sum oflag regressors. 
     Each col. corresponds to a k value in _k. entry i,j is given 
     by x_{t}+...+lrd_{t+1-_k_{j}}
 Note: missing values where time period falls out of data range
 ********/
proc(1) =mkkpslx(x,_k);
local xlags,kk,kpslx;
xlags = miss(zeros(rows(x),maxc(_k)),0); @cols are x{t},x{t-1},...,x{t+1-maxc(_k)}@
kk =1;
do until kk>maxc(_k);
  xlags[1+kk-1:rows(x),kk] = x[1:rows(x)-kk+1];  
  kk=kk+1;
endo;
kpslx =  miss(zeros(rows(x),rows(_k)),0);   @matrix of k period sum oflag regressors. 
     Each col. corresponds to a k value in _k. entry i,j is given by x_{t}+...+lrd_{t+1-_k_{j}}@
kk =1;
do until kk>rows(_k);
  kpslx[.,kk] = sumc(xlags[.,1:_k[kk]]');
  kk=kk+1;
endo;
retp(kpslx);
endp;

/***************************************************
********* proc to return the index in a vector
***************************************************/

proc indnvnew(what, where);
local i,ii,ind,indvec,dif;
indvec = zeros(rows(what),1);
i = 1;
do until i>rows(what);
  ii = 1;
  do until ii>rows(where);
  dif=abs(where[ii]-what[i]);

  if dif <= 0.0001;
  indvec[i] = ii;
  endif;
  ii = ii+1;
  endo;
i = i+1;
endo;
indvec = miss(indvec,0);
retp(indvec);

endp;

/**************************************************
**  proc to index the vector
**************************************************
** choose the one less or equal to the value computed
** vector where must be acendingly ordered
***************************************************/

proc indnvle(what, where);
local i,ii,ind,indvec,dif;
indvec = zeros(rows(what),1);

i = 1;
do until i>rows(what);
  if what[i]==where[1];
  indvec[i]=1;
  elseif what[i]==where[rows(where)];
  indvec[i]=rows(where);
  else;
  ii = 1;
  do until ii>rows(where);
     if where[ii]-what[i]>0;
     indvec[i]=ii-1;
     break;
     endif;
  ii = ii+1;
  endo;
  
  endif;
i = i+1;
endo;
indvec = miss(indvec,0);
retp(indvec);

endp;

/*  GE 9 june 92 procedure to estimate deltahat for the pt project
    ammended 7 June 93 to take median c as argument for ces project

Input:
        xt    :    x variable (potential I(1) variable)
        yt    :    independant variable in second stage regression
        cc    :    median estimate of c for xt
        nfirst:    starting observation (don't need to take into account
                    the number of lags in BIC as this is automatic)
        nlast :    final observation

Output:
        Dhat  :    the estimated delta hat value
 note : adjusted 13 april 94 to correct variance est in BIC - no df adjustment

*/

proc(2) = deltahat(xt,yt,nfirst,nlast,maxlag);

local dxa9, vno, nf, nl, bic, nlag, xa, i,
      xa9, amat, res, gtt, psel, omega, sig2,
      a1, omst, dhat;
dxa9=xt~yt;
vno=cols(dxa9);
/*stdc(dxa9);*/
/* choose lag length for var using SBIC   */
nf=nfirst+maxlag+1;
nl=nlast;

if maxlag==0;
else;
bic=zeros(maxlag,1);
nlag=1; do until nlag>maxlag;
xa=ones(nl-nf+1,1);
i=1;do until i > nlag;
 xa9=dxa9[nf-i:nl-i,.];
 xa=xa~xa9;
i=i+1; endo;
amat=inv(xa'xa)*xa'dxa9[nf:nl,.];
res=dxa9[nf:nl,.]-xa*amat;
gtt=ln(rows(xa))/(rows(xa));
bic[nlag,1]=ln(det((res'res)/(rows(xa))))+vno*nlag*gtt;

nlag=nlag+1; endo;
psel=minindc(bic);
endif;

/*  create A(1) and Sigma  */
if maxlag==0;
dxa9=dxa9-ones(rows(dxa9),vno)*meanc(dxa9);
omega=(dxa9[nf:nl,.]'dxa9[nf:nl,.])/(nl-nf+1);
else;
xa=ones(nl-nf+1,1);
i=1;do until i > psel;
 xa9=dxa9[nf-i:nl-i,.];
 xa=xa~xa9;
i=i+1; endo;
amat=inv(xa'xa)*xa'dxa9[nf:nl,.];
res=dxa9[nf:nl,.]-xa*amat;
sig2=(res'res)/(rows(xa));
i=1; do until i>psel;
if i==1;a1=amat[2:2+cols(xt),.];
else;
a1=a1+amat[2+vno*(i-1):2+vno*i-1,.];
endif;
i=i+1; endo;
a1=eye(vno)-a1;/* "a1  ";a1;*/
/*"sig2";sig2;    */

/* create Omega */
omega=(inv(a1')*sig2*inv(a1')');/*omega;*/
endif;

omst=omega.*eye(rows(omega));
omst=inv(sqrt(omst));
dhat=omst*omega*omst;


retp(dhat,omega);
endp;



@----------------------------------------------------@;
/* GE proc to do linear interp on a vector of values */

proc(1) = lint(x,yv,xv);
local xl, xu, ye, i, yl, yu;

/*   inputs :
        x    = 1x1 value between known points
        yv   = 2xk matrix (lower over upper) of value to be est at known points
        xv   = 2x1 vector (lower then upper) of value at known points
     output
        y    = y value corresponding to the x input
*/

    xl=xv[1,1];     xu=xv[2,1];
    ye=zeros(1,cols(yv));
    i=1; do until i>cols(yv);
    yl=yv[1,i];     yu=yv[2,i];
    ye[1,i]=yu - (yu-yl)*((xu-x)/(xu-xl));
    i=i+1; endo;
    retp(ye);
    endp;
@----------------------@

/* GE proc to select first stage size in the Bonferroni Procedure */

proc(1) = fstage(delta,a22);
local a1;
delta=abs(delta);

if a22==0.10; 
 if delta<0.5;
   {a1}=lint(delta,0.50|0.30, 0|0.5);
   elseif delta<0.7;
   {a1}=lint(delta,0.30|0.22,0.5|0.7);
   elseif delta<0.9;
   {a1}=lint(delta,0.22|0.13,0.7|0.9);
 
   else;
   {a1}=lint(delta,0.13|0.1,0.9|1);
  endif;
else;
  /*
  if delta<0.5;
   {a1}=lint(delta,0.50|0.34, 0|0.5);
   elseif delta<0.6;
   {a1}=lint(delta,0.34|0.23,0.5|0.6);
   elseif delta<0.7;
   {a1}=lint(delta,0.23|0.18,0.6|0.7);
   elseif delta<0.8;
   {a1}=lint(delta,0.18|0.14,0.7|0.8);
   elseif delta<0.9;
   {a1}=lint(delta,0.14|0.11,0.8|0.9);
 else;
   {a1}=lint(delta,0.11|0.1,0.9|1);
  endif;
  */

 /* new choices replication=20000 hor=1*/

if delta<0.5;
   {a1}=lint(delta,0.50|0.34, 0|0.5);
   elseif delta<0.6;
   {a1}=lint(delta,0.34|0.28,0.5|0.6);
   elseif delta<0.7;
   {a1}=lint(delta,0.28|0.22,0.6|0.7);
   elseif delta<0.8;
   {a1}=lint(delta,0.22|0.15,0.7|0.8);
   elseif delta<0.9;
   {a1}=lint(delta,0.15|0.11,0.8|0.9);
 else;
   {a1}=lint(delta,0.11|0.1,0.9|1);
  endif;


/******** optimal confidence levels when k=24 and T=200
**
if delta<0.5;
   {a1}=lint(delta,0.50|0.36, 0|0.5);
   elseif delta<0.7;
   {a1}=lint(delta,0.36|0.28,0.5|0.7);
   elseif delta<0.9;
   {a1}=lint(delta,0.28|0.21,0.7|0.9);
 
   else;
   {a1}=lint(delta,0.21|0.14,0.9|1);
  endif;

**********************************************************/
/**/





/* hor=10 
if delta<0.5;
   {a1}=lint(delta,0.50|0.18, 0|0.5);
   elseif delta<0.6;
   {a1}=lint(delta,0.18|0.12,0.5|0.6);
   elseif delta<0.7;
   {a1}=lint(delta,0.12|0.12,0.6|0.7);
   elseif delta<0.8;
   {a1}=lint(delta,0.12|0.12,0.7|0.8);
   elseif delta<0.9;
   {a1}=lint(delta,0.12|0.05,0.8|0.9);
 else;
   {a1}=lint(delta,0.04|0.04,0.9|1);
  endif;*/



endif;
a1=round(100*a1)/100;
if a1.>0.4; a1=0.4; endif;
retp(a1);
endp;

@-------------@

/* GE proc to select first stage size in the Bonferroni Procedure */

proc(1) = fstage2(delta,a22,k);
local a1;
delta=abs(delta);

if a22==0.10; 
 if delta<0.5;
   {a1}=lint(delta,0.50|0.30, 0|0.5);
   elseif delta<0.7;
   {a1}=lint(delta,0.30|0.22,0.5|0.7);
   elseif delta<0.9;
   {a1}=lint(delta,0.22|0.13,0.7|0.9);
 
   else;
   {a1}=lint(delta,0.13|0.1,0.9|1);
  endif;
else;
  /*
  if delta<0.5;
   {a1}=lint(delta,0.50|0.34, 0|0.5);
   elseif delta<0.6;
   {a1}=lint(delta,0.34|0.23,0.5|0.6);
   elseif delta<0.7;
   {a1}=lint(delta,0.23|0.18,0.6|0.7);
   elseif delta<0.8;
   {a1}=lint(delta,0.18|0.14,0.7|0.8);
   elseif delta<0.9;
   {a1}=lint(delta,0.14|0.11,0.8|0.9);
 else;
   {a1}=lint(delta,0.11|0.1,0.9|1);
  endif;
  */

 /* new choices replication=20000 hor=1*/
if k==1/200;
if delta<0.5;
   {a1}=lint(delta,0.50|0.34, 0|0.5);
   elseif delta<0.6;
   {a1}=lint(delta,0.34|0.28,0.5|0.6);
   elseif delta<0.7;
   {a1}=lint(delta,0.28|0.22,0.6|0.7);
   elseif delta<0.8;
   {a1}=lint(delta,0.22|0.15,0.7|0.8);
   elseif delta<0.9;
   {a1}=lint(delta,0.15|0.11,0.8|0.9);
 else;
   {a1}=lint(delta,0.11|0.1,0.9|1);
  endif;


/******** optimal confidence levels when k=24 and T=200
**/
elseif k==24/200;
if delta<0.5;
   {a1}=lint(delta,0.50|0.36, 0|0.5);
   elseif delta<0.7;
   {a1}=lint(delta,0.36|0.28,0.5|0.7);
   elseif delta<0.9;
   {a1}=lint(delta,0.28|0.21,0.7|0.9);
 
   else;
   {a1}=lint(delta,0.21|0.14,0.9|1);
  endif;

/**********************************************************/
/**/





/* hor=10 
if delta<0.5;
   {a1}=lint(delta,0.50|0.18, 0|0.5);
   elseif delta<0.6;
   {a1}=lint(delta,0.18|0.12,0.5|0.6);
   elseif delta<0.7;
   {a1}=lint(delta,0.12|0.12,0.6|0.7);
   elseif delta<0.8;
   {a1}=lint(delta,0.12|0.12,0.7|0.8);
   elseif delta<0.9;
   {a1}=lint(delta,0.12|0.05,0.8|0.9);
 else;
   {a1}=lint(delta,0.04|0.04,0.9|1);
  endif;*/



endif;
a1=round(100*a1)/100;
if a1.>0.4; a1=0.4; endif;
endif;
retp(a1);
endp;

@-------------@


/*   GE may93 mod of jun92 proc to use BIC to estimate lag
     length for a ADF test
*/

proc(2) = urbic(y,fdet,maxp);

/*  Inputs     :      y      - the data
                      fdet   - flag for deterministics
                      maxp   - maximum lag length
    Outputs    :      pstar  - BIC lag choice
                      lrsel  - Likelihood ratio choice
*/

local  n, bic, s2, x, maxl, sp, nlags, p, dy,
       z, yp, dyp, ip, zzi, zy, b, u, s, s2l,
       lr, lrsel, ii, pstar;

n=rows(y); bic=zeros(maxp,1); s2=zeros(maxp,1);
if fdet == 1; x=ones(n,1); elseif fdet == 2; x=ones(n,1)~seqa(1,1,n);
   else; x=zeros(n,1); endif;
maxl=fix(maxp*((n/100)^.33));
sp=1; do until sp>(maxp);
nlags=fix(sp*((n/100)^.33));p=nlags;
dy = 0|(y[2:n,1]-y[1:(n-1),1]);
z=y[(1+maxl):(n-1),1]; yp=y[(2+maxl):n,1]; dyp=dy[(2+maxl):n,1];
if sumc(sumc(x)).ne.0; z=z~x[(2+maxl):n,.]; endif;
ip=0; do until ip>=p;
  z=z~dy[(1+maxl-ip):(n-1-ip),1];
ip=ip+1; endo;
zzi=invpd(z'z); zy=z'yp;
b=zzi*zy;
u=yp-z*b; s=(u'u)/(rows(u)-cols(z));
s2[sp,1]=s;
bic[sp,1]=ln(s)+p*(ln(rows(u))/rows(u)); /*#### BIC VERSION #### */
sp=sp+1; endo;

z=y[(1+maxl):(n-1),1]; yp=y[(2+maxl):n,1]; dyp=dy[(2+maxl):n,1];
if sumc(sumc(x)).ne.0; z=z~x[(2+maxl):n,.]; endif;
zzi=invpd(z'z); zy=z'yp;
b=zzi*zy;
u=yp-z*b; s=(u'u);@/(rows(u)-cols(z));@
s2l=s|s2[1:maxp-1,1];
lr=rows(u)*((s2l-s2)./s2);
lrsel=0;
ii=maxp; do until ii<1;
if lr[ii,1]>2.71;
   lrsel=ii; ii=-1;
endif;
ii=ii-1;endo;


@if lrsel == 0; lrsel = 1; endif;@
pstar=minindc(bic);@  if pstar <3; pstar=3; endif;@
retp(pstar,lrsel);
endp;

/*
@ debug @
@rndseed 7192;@
fdet=1; nmc=100;
a1=zeros(nmc,4); a2=zeros(nmc,4);
i=1; do until i>nmc;
eps=rndn(201,1); eps=eps[2:201,1]+0.7*eps[1:200,1];
@eps1=recserar(eps,0,0.9);@
y=recserar(eps,0,1);
{a4,a1[i,.],a3,a2[i,.]}=dfs(y,fdet);
i=i+1; endo;
@meanc(a1.<-2.86);@
@meanc(a2);@
  */

@--------------------@

@ ------------------- Unit Root Test Statistics ---------------------------@

@ ADF                    JHS 4/20/90
 computes Dickey-Fuller test statistics
  y = data series, x = deterministic terms (matrix), p = no. of lags of diffs
@
proc(2)= ADF(y,fdet,p);
local n, x, nlags, dy, z, yp, ip, zzi, zy,
      b, u, s, ao, a1, nrhohat, c, what, t;
n=rows(y);
if fdet == 1; x=ones(n,1); elseif fdet == 2; x=ones(n,1)~seqa(1,1,n);
   else; x=zeros(n,1); endif;
if p<0; nlags=fix(4*((n/100)^.33)); endif;
dy = 0|(y[2:n,1]-y[1:(n-1),1]);
z=y[(1+p):(n-1),1]; yp=y[(2+p):n,1]-y[(1+p):(n-1),1];
if sumc(sumc(x)).ne.0; z=z~x[(2+p):n,.]; endif;
ip=0; do until ip>=p;
  z=z~dy[(1+p-ip):(n-1-ip),1];
ip=ip+1; endo;
zzi=invpd(z'z); zy=z'yp;
b=zzi*zy;
u=yp-z*b; s=(u'u)/(rows(u)-cols(z));
if sumc(sumc(x)).ne.0; ao=sumc(b[1:1+cols(x),1]); else; ao=b[1,1]; endif;
a1=sumc(b)-ao;
nrhohat=rows(u)*(b[1,1])/(1-a1);
c=b[1,1];
what=s/((1-a1)*(1-a1));
t=(b[1,1])/sqrt(s*zzi[1,1]);
retp(c|t,what);
endp;

@--------------------@

@ unitcidf.prc  JHS 9/7/90 (modified 9/18/91)
   proc to compute confidence insterval for local-to-unit root parameter c,
    given a value of a ADF test statistic.
   Calculations and theory are described in J. Stock, "Confidence Intervals
   for the Largest Autoregressive Root ..."
   Note that these intervals combine disjoint confidence intervals when the
   correct interval would be disjoint (this simplifies the algorithm).
@
proc(1) = unitci(tdf,clevel,cv);
@
 tdf = value of test statistic
  clevel = desired significance level for test -- so the confidence interval
           will be 100(1-clevel)%
  cv = matrix of tabulated critical values read in fmt form
 output: 2-dim vector (c0, c1) -- the 100*clevel% confidence interval for c
        based on critical values in the lookup table saved as the matrix cv.
@
 local
        alpha0, sel0, sel1, i0, i1, cc, cv0, cv1, d1, d2, dd, jj, j0, j1,
        c0, c1, wt0, wt1;

 alpha0=0.5*(1-clevel);
 sel0=(fix(1000*cv[1,.]'+.01).==fix(1000*alpha0+.01));
 sel1=(fix(1000*cv[1,.]'+.01).==fix(1000*(1-alpha0)+.01));
 i0=sumc(seqa(1,1,cols(cv)).*sel0);
 i1=sumc(seqa(1,1,cols(cv)).*sel1);
 cc=cv[2:rows(cv),1]; cv0=cv[2:rows(cv),i0]; cv1=cv[2:rows(cv),i1];
 d1=(cv0.<tdf); d2=(cv1.>tdf); dd=d1.*d2;
 if i0==i1;    @ return median -- 0-width CI @
  if sumc(d1)>0 and sumc(d2)>0;
   jj=selif(seqa(1,1,rows(cc)),d1); j0=maxc(jj); j1=j0+1;
  elseif sumc(d1)==0; c0=-99.99; c1=c0; goto done;
  elseif sumc(d2)==0; c0=99.99; c1=c0; goto done;
  endif;
 elseif sumc(dd)>0; @ nonempty CI @
  jj=selif(seqa(1,1,rows(cc)),dd); j0=maxc(jj); j1=minc(jj);
 else; @ empty confidence interval -- outside the range covered by cv @
  if sumc(d1)==0; c0=-99.99; c1=-99.99;
  elseif sumc(d2)==0; c0=99.99; c1=99.99;
  endif;
  goto done;
 endif;
 if j0<rows(cc);
  wt0=(tdf-cv0[j0,1])/(cv0[j0+1,1]-cv0[j0,1]);
   c0=(1-wt0)*cc[j0,1] + wt0*cc[j0+1,1];
 else; wt0=0; c0=99.99; endif;
 if j1>1;
  wt1=(tdf-cv1[j1-1,1])/(cv1[j1,1]-cv1[j1-1,1]);
   c1=(1-wt1)*cc[j1-1,1] + wt1*cc[j1,1];
 else; wt1=0; c1=-99.99; endif;
done:
retp(c1|c0);
endp;

/**** may not use this updated version, since the critical values
for bonforroni approach were generated using the old ones*****/

@***
mkyk: make k-period return (y) from one period return
inputs:
  _k: column vector or return horizons or length
  y: N by 1 column vector of returns
outputs:
  kpy: k-period returns
         An N by rows(_k) matrix
         kpy[t,j] = (y_{t+1})+...+y{t+_k[j]})
****
proc(1) = mkyk(y,_k);
local yleads,y_k,kk,kpy;
/*form k- period nominal log return*/
yleads = miss(zeros(rows(y),maxc(_k)),0); /*cols are y_{k+1},lrd_{k+2},...,y_{t+masc(_k)*/
y_k =  miss(zeros(rows(y),rows(_k)),0); /*matrix of k period returns. Each col. corresponds
                                           to a k value in _k. entry i,j is given by
					   y_{t+1}+...+y_{t+_k_{j}}*/



kk =1;
do until kk>maxc(_k);
  yleads[1:rows(y)-kk+1,kk] = y[1+kk-1:rows(y)];  
  kk=kk+1;
endo;
kk =1;
do until kk>rows(_k);
  y_k[.,kk] = sumc(yleads[.,1:_k[kk]]');
  kk=kk+1;
endo;
kpy = y_k;

retp(kpy);
endp;
@
@**************@



             



  
  
    









